import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter/cupertino.dart';
import 'dart:async';

void main() {
  runApp(MyApp());
}

class MyApp extends StatelessWidget {
  // This widget is the root of your application.
  @override
  Widget build(BuildContext context) {
    String title = 'method channel app';
    return MaterialApp(
      title: 'Flutter Demo',
      theme: ThemeData(
        primarySwatch: Colors.purple,
      ),
      home: new MyHomePage(title),
    );
  }
}

class MyHomePage extends StatefulWidget {
  final String title;
  MyHomePage(this.title);
  @override
  _MyHomePageState createState() => _MyHomePageState();
}

class _MyHomePageState extends State<MyHomePage> {
  static const platformMethodChannel = const MethodChannel('com.sicilyliu');
  String nativeMess = "";
  String helloCountStr = "";
  num flutterHelloCount = 0;
  num nativeHelloCount = 0;

  Future<dynamic> getData(String method, {Map arguments}) async {
    if (arguments == null) {
      return await platformMethodChannel.invokeMethod(method);
    } else {
      return await platformMethodChannel.invokeMethod(method, arguments);
    }
  }

  Future<dynamic> nativeMessageListener() async {
    platformMethodChannel.setMethodCallHandler((resultCall) {
      MethodCall call = resultCall;
      String method = call.method;
      Map arguments = call.arguments;
      int code = arguments["code"];
      String message = arguments["message"];
      if (method == "hello") {
        nativeHelloCount++;
      }
      setState(() {
        helloCountStr =
            'flutter hello = $flutterHelloCount\n native hello = $nativeHelloCount';
        nativeMess =
            " native calling method 【$method】, code 【$code】 message 【$message】";
      });
    });
  }

  @override
  void initState() {
    nativeMessageListener();
  }

  @override
  Widget build(BuildContext context) {
    return new Scaffold(
      appBar: new AppBar(
        title: new Text(widget.title),
      ),
      body: new Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          Padding(
            padding: EdgeInsets.all(30),
            child: new Text(
              '$nativeMess',
              style: TextStyle(
                color: Colors.blue,
                fontSize: 18,
              ),
            ),
          ),
          Padding(
            padding: EdgeInsets.all(30),
            child: new Text(
              '$helloCountStr',
              style: TextStyle(
                color: Colors.blue[300],
                fontSize: 20,
              ),
            ),
          ),
          Padding(
            padding: EdgeInsets.all(30),
            child: FlatButton(
                onPressed: () {
                  getData('battery')
                    ..then((value) {
                      setState(() {
                        helloCountStr =
                            'flutter hello = $flutterHelloCount\n, native hello = $nativeHelloCount';
                        nativeMess = 'battery = $value';
                      });
                    });
                },
                child: new Text(
                  'get native battery',
                  style: TextStyle(
                    color: Colors.orange,
                    fontSize: 20,
                  ),
                )),
          ),
          Padding(
            padding: EdgeInsets.all(30),
            child: FlatButton(
                onPressed: () {
                  flutterHelloCount++;
                  getData("hello")
                    ..then((dynamic value) {
                      int code = value["code"];
                      String message = value["message"];
                      setState(() {
                        helloCountStr =
                            'flutter hello = $flutterHelloCount\n, native hello = $nativeHelloCount';
                        nativeMess = 'code : $code, message: $message';
                      });
                    });
                },
                child: new Text(
                  'say hello to native',
                  style: TextStyle(
                    color: Colors.green,
                    fontSize: 20,
                  ),
                )),
          ),
          Padding(
            padding: EdgeInsets.all(30),
            child: FlatButton(
              onPressed: () {
                getData("native hello");
              },
              child: new Text(
                'let native say hello',
                style: TextStyle(
                  color: Colors.brown,
                  fontSize: 20,
                ),
              ),
            ),
          ),
        ],
      ),
    );
  }
}
